@@ -19,19 +19,8 @@
  *
  */
 
+#include "tvheadend/utilities/Logger.h"
 #include "Tvheadend.h"
-#include "client.h"
-
-#include "platform/threads/mutex.h"
-#include "platform/threads/atomics.h"
-#include "platform/util/timeutils.h"
-#include "platform/sockets/tcp.h"
-
-extern "C" {
-#include "libhts/htsmsg_binary.h"
-#include "libhts/sha1.h"
-}
-
 #include "xbmc_codec_descriptor.hpp"
 
 #define TVH_TO_DVD_TIME(x) ((double)x * DVD_TIME_BASE / 1000000.0)
@@ -39,25 +28,34 @@ extern "C" {
 using namespace std;
 using namespace ADDON;
 using namespace PLATFORM;
+using namespace tvheadend;
+using namespace tvheadend::utilities;
 
 CHTSPDemuxer::CHTSPDemuxer ( CHTSPConnection &conn )
   : m_conn(conn), m_pktBuffer((size_t)-1),
-    m_seekTime(INVALID_SEEKTIME)
+    m_seekTime(INVALID_SEEKTIME),
+    m_subscription(conn),
+    m_seeking(false), m_speedChange(false)
 {
+  m_lastUse = 0;
 }
 
-CHTSPDemuxer::~CHTSPDemuxer ( void )
+CHTSPDemuxer::~CHTSPDemuxer ()
 {
 }
 
 void CHTSPDemuxer::Connected ( void )
 {
   /* Re-subscribe */
-  if (m_subscription.active)
+  if (m_subscription.IsActive())
   {
-    tvhdebug("demux re-starting stream");
-    SendSubscribe(true);
-    SendSpeed(true);
+    Logger::Log(LogLevel::LEVEL_DEBUG, "demux re-starting stream");
+    m_subscription.SendSubscribe(0, 0, true);
+    m_subscription.SendSpeed(0, true);
+
+    /* Reset status */
+    m_signalInfo.Clear();
+    m_sourceInfo.Clear();
   }
 }
 
@@ -68,8 +66,8 @@ void CHTSPDemuxer::Connected ( void )
 void CHTSPDemuxer::Close0 ( void )
 {
   /* Send unsubscribe */
-  if (m_subscription.active)
-    SendUnsubscribe();
+  if (m_subscription.IsActive())
+    m_subscription.SendUnsubscribe();
 
   /* Clear */
   Flush();
@@ -81,47 +79,52 @@ void CHTSPDemuxer::Abort0 ( void )
   CLockObject lock(m_mutex);
   m_streams.Clear();
   m_streamStat.clear();
+  m_seeking = false;
+  m_speedChange = false;
 }
 
 
-bool CHTSPDemuxer::Open ( const PVR_CHANNEL &chn )
+bool CHTSPDemuxer::Open ( uint32_t channelId, enum eSubscriptionWeight weight )
 {
   CLockObject lock(m_conn.Mutex());
-  tvhdebug("demux open");
+  Logger::Log(LogLevel::LEVEL_DEBUG, "demux open");
 
   /* Close current stream */
   Close0();
-  
-  /* Create new subscription */
-  m_subscription = SSubscription();
-  m_subscription.channelId = chn.iUniqueId;
 
-  /* Open */
-  SendSubscribe();
+  /* Open new subscription */
+  m_subscription.SendSubscribe(channelId, weight);
   
+  /* Reset status */
+  m_signalInfo.Clear();
+  m_sourceInfo.Clear();
+
   /* Send unsubscribe if subscribing failed */
-  if (!m_subscription.active)
-    SendUnsubscribe();
+  if (!m_subscription.IsActive())
+    m_subscription.SendUnsubscribe();
+  else
+    m_lastUse = time(NULL);
   
-  return m_subscription.active;
+  return m_subscription.IsActive();
 }
 
 void CHTSPDemuxer::Close ( void )
 {
   CLockObject lock(m_conn.Mutex());
   Close0();
-  tvhdebug("demux close");
+  Logger::Log(LogLevel::LEVEL_DEBUG, "demux close");
 }
 
 DemuxPacket *CHTSPDemuxer::Read ( void )
 {
   DemuxPacket *pkt = NULL;
+  m_lastUse = time(NULL);
   if (m_pktBuffer.Pop(pkt, 1000)) {
-    tvhtrace("demux read idx :%d pts %lf len %lld",
+    Logger::Log(LogLevel::LEVEL_TRACE, "demux read idx :%d pts %lf len %lld",
              pkt->iStreamId, pkt->pts, (long long)pkt->iSize);
     return pkt;
   }
-  tvhtrace("demux read nothing");
+  Logger::Log(LogLevel::LEVEL_TRACE, "demux read nothing");
   
   return PVR->AllocateDemuxPacket(0);
 }
@@ -129,14 +132,26 @@ DemuxPacket *CHTSPDemuxer::Read ( void )
 void CHTSPDemuxer::Flush ( void )
 {
   DemuxPacket *pkt;
-  tvhtrace("demux flush");
+  Logger::Log(LogLevel::LEVEL_TRACE, "demux flush");
   while (m_pktBuffer.Pop(pkt))
     PVR->FreeDemuxPacket(pkt);
 }
 
+void CHTSPDemuxer::Trim ( void )
+{
+  DemuxPacket *pkt;
+
+  Logger::Log(LogLevel::LEVEL_TRACE, "demux trim");
+  /* reduce used buffer space to what is needed for DVDPlayer to resume
+   * playback without buffering. This depends on the bitrate, so we don't set
+   * this too small. */
+  while (m_pktBuffer.Size() > 512 && m_pktBuffer.Pop(pkt))
+    PVR->FreeDemuxPacket(pkt);
+}
+
 void CHTSPDemuxer::Abort ( void )
 {
-  tvhtrace("demux abort");
+  Logger::Log(LogLevel::LEVEL_TRACE, "demux abort");
   CLockObject lock(m_conn.Mutex());
   Abort0();
 }
@@ -144,40 +159,35 @@ void CHTSPDemuxer::Abort ( void )
 bool CHTSPDemuxer::Seek 
   ( int time, bool _unused(backwards), double *startpts )
 {
-  htsmsg_t *m;
-
-  CLockObject lock(m_conn.Mutex());
-  if (!m_subscription.active)
+  if (!m_subscription.IsActive())
     return false;
 
-  tvhdebug("demux seek %d", time);
-
-  /* Build message */
-  m = htsmsg_create_map();  
-  htsmsg_add_u32(m, "subscriptionId", m_subscription.subscriptionId);
-  htsmsg_add_s64(m, "time",           (int64_t)time * 1000LL);
-  htsmsg_add_u32(m, "absolute",       1);
-
-  /* Send and Wait */
-  m = m_conn.SendAndWait("subscriptionSeek", m);
-  if (!m)
+  m_seekTime = 0;
+  m_seeking = true;
+  if (!m_subscription.SendSeek(time)) {
+    m_seeking = false;
     return false;
-  
-  htsmsg_destroy(m);
+  }
 
   /* Wait for time */
-  if (!m_seekCond.Wait(m_conn.Mutex(), m_seekTime, tvh->GetSettings().iResponseTimeout))
+  CLockObject lock(m_conn.Mutex());
+
+  m_seekTime = 0;
+  if (!m_seekCond.Wait(m_conn.Mutex(), m_seekTime, Settings::GetInstance().GetResponseTimeout()))
   {
-    tvherror("failed to get subscriptionSeek response");
+    Logger::Log(LogLevel::LEVEL_ERROR, "failed to get subscriptionSeek response");
+    m_seeking = false;
+    Flush(); /* try to resync */
     return false;
   }
   
+  m_seeking = false;
   if (m_seekTime == INVALID_SEEKTIME)
     return false;
 
   /* Store */
-  *startpts = TVH_TO_DVD_TIME(m_seekTime);
-  tvhtrace("demux seek startpts = %lf", *startpts);
+  *startpts = TVH_TO_DVD_TIME(m_seekTime - 1);
+  Logger::Log(LogLevel::LEVEL_TRACE, "demux seek startpts = %lf", *startpts);
 
   return true;
 }
@@ -185,10 +195,20 @@ bool CHTSPDemuxer::Seek
 void CHTSPDemuxer::Speed ( int speed )
 {
   CLockObject lock(m_conn.Mutex());
-  if (!m_subscription.active)
+  if (!m_subscription.IsActive())
     return;
-  m_subscription.speed = speed;
-  SendSpeed();
+  if (speed != m_subscription.GetSpeed() && (speed < 0 || speed >= 4000)) {
+    m_speedChange = true;
+    Flush();
+  }
+  m_subscription.SendSpeed(speed);
+}
+
+void CHTSPDemuxer::Weight ( enum eSubscriptionWeight weight )
+{
+  if (!m_subscription.IsActive() || m_subscription.GetWeight() == static_cast<uint32_t>(weight))
+    return;
+  m_subscription.SendWeight(static_cast<uint32_t>(weight));
 }
 
 PVR_ERROR CHTSPDemuxer::CurrentStreams ( PVR_STREAM_PROPERTIES *streams )
@@ -223,79 +243,9 @@ PVR_ERROR CHTSPDemuxer::CurrentSignal ( PVR_SIGNAL_STATUS &sig )
   return PVR_ERROR_NO_ERROR;
 }
 
-/* **************************************************************************
- * Send Messages
- * *************************************************************************/
-
-void CHTSPDemuxer::SendSubscribe ( bool force )
+void CHTSPDemuxer::SetStreamingProfile(const std::string &profile)
 {
-  htsmsg_t *m;
-
-  /* Reset status */
-  m_signalInfo.Clear();
-  m_sourceInfo.Clear();
-
-  /* Build message */
-  m = htsmsg_create_map();
-  htsmsg_add_s32(m, "channelId",       m_subscription.channelId);
-  htsmsg_add_u32(m, "subscriptionId",  m_subscription.subscriptionId);
-  htsmsg_add_u32(m, "timeshiftPeriod", (uint32_t)~0);
-  htsmsg_add_u32(m, "normts",          1);
-  htsmsg_add_u32(m, "queueDepth",      2000000);
-
-  /* Send and Wait for response */
-  tvhdebug("demux subscribe to %d", m_subscription.channelId);
-  if (force)
-    m = m_conn.SendAndWait0("subscribe", m);
-  else
-    m = m_conn.SendAndWait("subscribe", m);
-  if (m == NULL)
-    return;
-
-  htsmsg_destroy(m);
-
-  m_subscription.active = true;
-  tvhdebug("demux successfully subscribed to %d", m_subscription.channelId);
-}
-
-void CHTSPDemuxer::SendUnsubscribe ( void )
-{
-  htsmsg_t *m;
-
-  /* Build message */
-  m = htsmsg_create_map();
-  htsmsg_add_u32(m, "subscriptionId", m_subscription.subscriptionId);
-
-  /* Mark subscription as inactive immediately in case this command fails */
-  m_subscription.active = false;
-  
-  /* Send and Wait */
-  tvhdebug("demux unsubscribe from %d", m_subscription.channelId);
-  if ((m = m_conn.SendAndWait("unsubscribe", m)) == NULL)
-    return;
-
-  htsmsg_destroy(m);
-  tvhdebug("demux successfully unsubscribed %d", m_subscription.channelId);
-}
-
-void CHTSPDemuxer::SendSpeed ( bool force )
-{
-  htsmsg_t *m;
-  int speed = m_subscription.speed / 10; // XBMC uses values an order of magnitude larger than tvheadend
-
-  /* Build message */
-  m = htsmsg_create_map();
-  htsmsg_add_u32(m, "subscriptionId", m_subscription.subscriptionId);
-  htsmsg_add_s32(m, "speed",          speed);
-  tvhdebug("demux send speed %d", speed);
-
-  /* Send and Wait */
-  if (force)
-    m = m_conn.SendAndWait0("subscriptionSpeed", m);
-  else
-    m = m_conn.SendAndWait("subscriptionSpeed", m);
-  if (m)
-    htsmsg_destroy(m);
+  m_subscription.SetProfile(profile);
 }
 
 /* **************************************************************************
@@ -304,23 +254,13 @@ void CHTSPDemuxer::SendSpeed ( bool force )
 
 bool CHTSPDemuxer::ProcessMessage ( const char *method, htsmsg_t *m )
 {
-  uint32_t subId;
-
   CLockObject lock(m_mutex);
 
-  /* No subscriptionId - not for demuxer */
-  if (htsmsg_get_u32(m, "subscriptionId", &subId))
-    return false;
-
-  /* Not current subscription - ignore */
-  else if (subId != m_subscription.subscriptionId)
-    return true;
-
   /* Subscription messages */
-  else if (!strcmp("muxpkt", method))
+  if (!strcmp("muxpkt", method))
     ParseMuxPacket(m);
   else if (!strcmp("subscriptionStatus", method))
-    ParseSubscriptionStatus(m);
+    m_subscription.ParseSubscriptionStatus(m);
   else if (!strcmp("queueStatus", method))
     ParseQueueStatus(m);
   else if (!strcmp("signalStatus", method))
@@ -336,7 +276,7 @@ bool CHTSPDemuxer::ProcessMessage ( const char *method, htsmsg_t *m )
   else if (!strcmp("subscriptionSpeed", method))
     ParseSubscriptionSpeed(m);
   else
-    tvhdebug("demux unhandled subscription message [%s]",
+    Logger::Log(LogLevel::LEVEL_DEBUG, "demux unhandled subscription message [%s]",
               method);
 
   return true;
@@ -350,20 +290,20 @@ void CHTSPDemuxer::ParseMuxPacket ( htsmsg_t *m )
   size_t      binlen;
   DemuxPacket *pkt;
   char        _unused(type) = 0;
-  int         iStreamId;
+  int         iStreamId, ignore;
   
   /* Ignore packets while switching channels */
-  if (!m_subscription.active)
+  if (!m_subscription.IsActive())
   {
-    tvhdebug("Ignored mux packet due to channel switch");
+    Logger::Log(LogLevel::LEVEL_DEBUG, "Ignored mux packet due to channel switch");
     return;
   }
   
   /* Validate fields */
   if (htsmsg_get_u32(m, "stream", &idx) ||
       htsmsg_get_bin(m, "payload", &bin, &binlen))
-  { 
-    tvherror("malformed muxpkt: 'stream'/'payload' missing");
+  {
+    Logger::Log(LogLevel::LEVEL_ERROR, "malformed muxpkt: 'stream'/'payload' missing");
     return;
   }
 
@@ -373,7 +313,7 @@ void CHTSPDemuxer::ParseMuxPacket ( htsmsg_t *m )
   /* Drop packets for unknown streams */
   if (-1 == (iStreamId = m_streams.GetStreamId(idx)))
   {
-    tvhdebug("Dropped packet with unknown stream index %i", idx);
+    Logger::Log(LogLevel::LEVEL_DEBUG, "Dropped packet with unknown stream index %i", idx);
     return;
   }
   
@@ -405,11 +345,17 @@ void CHTSPDemuxer::ParseMuxPacket ( htsmsg_t *m )
   if (!type)
     type = '_';
 
-  tvhtrace("demux pkt idx %d:%d type %c pts %lf len %lld",
-           idx, pkt->iStreamId, type, pkt->pts, (long long)binlen);
+  ignore = m_seeking || m_speedChange;
+
+  Logger::Log(LogLevel::LEVEL_TRACE, "demux pkt idx %d:%d type %c pts %lf len %lld%s",
+           idx, pkt->iStreamId, type, pkt->pts, (long long)binlen,
+           ignore ? " IGNORE" : "");
 
   /* Store */
-  m_pktBuffer.Push(pkt);
+  if (!ignore)
+    m_pktBuffer.Push(pkt);
+  else
+    PVR->FreeDemuxPacket(pkt);
 }
 
 void CHTSPDemuxer::ParseSubscriptionStart ( htsmsg_t *m )
@@ -422,7 +368,7 @@ void CHTSPDemuxer::ParseSubscriptionStart ( htsmsg_t *m )
   /* Validate */
   if ((l = htsmsg_get_list(m, "streams")) == NULL)
   {
-    tvherror("malformed subscriptionStart: 'streams' missing");
+    Logger::Log(LogLevel::LEVEL_ERROR, "malformed subscriptionStart: 'streams' missing");
     return;
   }
   m_streamStat.clear();
@@ -444,7 +390,7 @@ void CHTSPDemuxer::ParseSubscriptionStart ( htsmsg_t *m )
     /* Find stream */
     m_streamStat[idx] = 0;
     m_streams.GetStreamData(idx, &stream);
-    tvhdebug("demux subscription start");
+    Logger::Log(LogLevel::LEVEL_DEBUG, "demux subscription start");
     
     CodecDescriptor codecDescriptor = CodecDescriptor::GetCodecByName(type);
     xbmc_codec_t codec = codecDescriptor.Codec();
@@ -496,7 +442,7 @@ void CHTSPDemuxer::ParseSubscriptionStart ( htsmsg_t *m )
            some versions of tvheadend and is here for backward compatibility. */
         if (stream.iWidth == 0 || stream.iHeight == 0)
         {
-          tvhinfo("Ignoring subscriptionStart, stream details missing");
+          Logger::Log(LogLevel::LEVEL_DEBUG, "Ignoring subscriptionStart, stream details missing");
           return;
         }
         
@@ -511,12 +457,12 @@ void CHTSPDemuxer::ParseSubscriptionStart ( htsmsg_t *m )
       }
         
       streams.push_back(stream);
-      tvhdebug("  id: %d, type %s, codec: %u", idx, type, stream.iCodecId);
+      Logger::Log(LogLevel::LEVEL_DEBUG, "  id: %d, type %s, codec: %u", idx, type, stream.iCodecId);
     }
   }
 
   /* Update streams */
-  tvhdebug("demux stream change");
+  Logger::Log(LogLevel::LEVEL_DEBUG, "demux stream change");
   m_streams.UpdateStreams(streams);
   pkt = PVR->AllocateDemuxPacket(0);
   pkt->iStreamId = DMX_SPECIALID_STREAMCHANGE;
@@ -532,42 +478,42 @@ void CHTSPDemuxer::ParseSourceInfo ( htsmsg_t *m )
   
   /* Ignore */
   if (!m) return;
-  
-  tvhtrace("demux sourceInfo:");
+
+  Logger::Log(LogLevel::LEVEL_TRACE, "demux sourceInfo:");
 
   /* include position in mux name
    * as users might receive multiple satellite positions */
   m_sourceInfo.si_mux.clear();
   if ((str = htsmsg_get_str(m, "satpos")) != NULL)
   {
-    tvhtrace("  satpos : %s", str);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  satpos : %s", str);
     m_sourceInfo.si_mux.append(str);
     m_sourceInfo.si_mux.append(": ");
   }
   if ((str = htsmsg_get_str(m, "mux")) != NULL)
   {
-    tvhtrace("  mux     : %s", str);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  mux     : %s", str);
     m_sourceInfo.si_mux.append(str);
   }
 
   if ((str = htsmsg_get_str(m, "adapter")) != NULL)
   {
-    tvhtrace("  adapter : %s", str);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  adapter : %s", str);
     m_sourceInfo.si_adapter  = str;
   }
   if ((str = htsmsg_get_str(m, "network")) != NULL)
   {
-    tvhtrace("  network : %s", str);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  network : %s", str);
     m_sourceInfo.si_network  = str;
   }
   if ((str = htsmsg_get_str(m, "provider")) != NULL)
   {
-    tvhtrace("  provider : %s", str);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  provider : %s", str);
     m_sourceInfo.si_provider = str;
   }
   if ((str = htsmsg_get_str(m, "service")) != NULL)
   {
-    tvhtrace("  service : %s", str);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  service : %s", str);
     m_sourceInfo.si_service  = str;
   }
 }
@@ -583,8 +529,10 @@ void CHTSPDemuxer::ParseSubscriptionSkip ( htsmsg_t *m )
   if (htsmsg_get_s64(m, "time", &s64)) {
     m_seekTime = INVALID_SEEKTIME;
   } else {
-    m_seekTime = s64;
+    m_seekTime = s64 < 0 ? 1 : s64 + 1; /* it must not be zero! */
+    Flush(); /* flush old packets (with wrong pts) */
   }
+  m_seeking = false;
   m_seekCond.Broadcast();
 }
 
@@ -592,19 +540,10 @@ void CHTSPDemuxer::ParseSubscriptionSpeed ( htsmsg_t *m )
 {
   uint32_t u32;
   if (!htsmsg_get_u32(m, "speed", &u32))
-    tvhtrace("recv speed %d", u32);
-}
-
-void CHTSPDemuxer::ParseSubscriptionStatus ( htsmsg_t *m )
-{
-  const char *status;
-  status = htsmsg_get_str(m, "status");
-
-  // this field is absent when everything is fine
-  if (status != NULL)
-  {
-    tvhinfo("Bad subscription status: %s", status);
-    XBMC->QueueNotification(QUEUE_INFO, status);
+    Logger::Log(LogLevel::LEVEL_TRACE, "recv speed %d", u32);
+  if (m_speedChange) {
+    Flush();
+    m_speedChange = false;
   }
 }
 
@@ -612,23 +551,23 @@ void CHTSPDemuxer::ParseQueueStatus ( htsmsg_t *_unused(m) )
 {
   uint32_t u32;
   map<int,int>::const_iterator it;
-  tvhtrace("stream stats:");
+  Logger::Log(LogLevel::LEVEL_TRACE, "stream stats:");
   for (it = m_streamStat.begin(); it != m_streamStat.end(); ++it)
-    tvhtrace("  idx:%d num:%d", it->first, it->second);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  idx:%d num:%d", it->first, it->second);
 
-  tvhtrace("queue stats:");
+  Logger::Log(LogLevel::LEVEL_TRACE, "queue stats:");
   if (!htsmsg_get_u32(m, "packets", &u32))
-    tvhtrace("  pkts  %d", u32);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  pkts  %d", u32);
   if (!htsmsg_get_u32(m, "bytes", &u32))
-    tvhtrace("  bytes %d", u32);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  bytes %d", u32);
   if (!htsmsg_get_u32(m, "delay", &u32))
-    tvhtrace("  delay %d", u32);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  delay %d", u32);
   if (!htsmsg_get_u32(m, "Idrops", &u32))
-    tvhtrace("  Idrop %d", u32);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  Idrop %d", u32);
   if (!htsmsg_get_u32(m, "Pdrops", &u32))
-    tvhtrace("  Pdrop %d", u32);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  Pdrop %d", u32);
   if (!htsmsg_get_u32(m, "Bdrops", &u32))
-    tvhtrace("  Bdrop %d", u32);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  Bdrop %d", u32);
 }
 
 void CHTSPDemuxer::ParseSignalStatus ( htsmsg_t *m )
@@ -640,34 +579,34 @@ void CHTSPDemuxer::ParseSignalStatus ( htsmsg_t *m )
   m_signalInfo.Clear();
 
   /* Parse */
-  tvhtrace("signalStatus:");
+  Logger::Log(LogLevel::LEVEL_TRACE, "signalStatus:");
   if ((str = htsmsg_get_str(m, "feStatus")) != NULL)
   {
-    tvhtrace("  status : %s", str);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  status : %s", str);
     m_signalInfo.fe_status = str;
   }
   else
   {
-    tvherror("malformed signalStatus: 'feStatus' missing, ignoring");
+    Logger::Log(LogLevel::LEVEL_ERROR, "malformed signalStatus: 'feStatus' missing, ignoring");
   }
   if (!htsmsg_get_u32(m, "feSNR", &u32))
   {
-    tvhtrace("  snr    : %d", u32);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  snr    : %d", u32);
     m_signalInfo.fe_snr    = u32;
   }
   if (!htsmsg_get_u32(m, "feBER", &u32))
   {
-    tvhtrace("  ber    : %d", u32);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  ber    : %d", u32);
     m_signalInfo.fe_ber    = u32;
   }
   if (!htsmsg_get_u32(m, "feUNC", &u32))
   {
-    tvhtrace("  unc    : %d", u32);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  unc    : %d", u32);
     m_signalInfo.fe_unc    = u32;
   }
   if (!htsmsg_get_u32(m, "feSignal", &u32))
   {
-    tvhtrace("  signal    : %d", u32);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  signal    : %d", u32);
     m_signalInfo.fe_signal = u32;
   }
 }
@@ -678,33 +617,33 @@ void CHTSPDemuxer::ParseTimeshiftStatus ( htsmsg_t *m )
   int64_t s64;
 
   /* Parse */
-  tvhtrace("timeshiftStatus:");
+  Logger::Log(LogLevel::LEVEL_TRACE, "timeshiftStatus:");
   if (!htsmsg_get_u32(m, "full", &u32))
   {
-    tvhtrace("  full  : %d", m_timeshiftStatus.full);
-    m_timeshiftStatus.full = (bool)u32;
+    Logger::Log(LogLevel::LEVEL_TRACE, "  full  : %d", u32);
+    m_timeshiftStatus.full = u32 == 0 ? false : true;
   }
   else
   {
-    tvherror("malformed timeshiftStatus: 'full' missing, ignoring");
+    Logger::Log(LogLevel::LEVEL_ERROR, "malformed timeshiftStatus: 'full' missing, ignoring");
   }
   if (!htsmsg_get_s64(m, "shift", &s64))
   {
-    tvhtrace("  shift : %lld", m_timeshiftStatus.shift);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  shift : %lld", s64);
     m_timeshiftStatus.shift = s64;
   }
   else
   {
-    tvherror("malformed timeshiftStatus: 'shift' missing, ignoring");
+    Logger::Log(LogLevel::LEVEL_ERROR, "malformed timeshiftStatus: 'shift' missing, ignoring");
   }
   if (!htsmsg_get_s64(m, "start", &s64))
   {
-    tvhtrace("  start : %lld", m_timeshiftStatus.start);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  start : %lld", s64);
     m_timeshiftStatus.start = s64;
   }
   if (!htsmsg_get_s64(m, "end", &s64))
   {
-    tvhtrace("  end   : %lld", m_timeshiftStatus.end);
+    Logger::Log(LogLevel::LEVEL_TRACE, "  end   : %lld", s64);
     m_timeshiftStatus.end = s64;
   }
 }
